C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE GLOBAL_CC      *****                *
C  *                                                                          *
C  *    CROSS-CORRELATE POLAR FFT DATA TO DETERMINE AND REFINE PARTICLE       *
C  *     ORIENTATIONS (GLOBAL MODE VERSION).                                  *
C  *                                                                          *
C  *    LOGICAL UNITS: 1 = REFERENCE 3D MAP                                   *
C  *                   2 = INPUT PARAMETERS DATA FILE(s)                      *
C  *                   3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS)             *
C  *                   4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS)             *
C  *                   7 = INPUT RAW IMAGE DATA                               *
C  *                   8 = STATS OUTPUT FILE (EMPFT.RES1)                     *
C  *                   9 = STATS OUTPUT FILE (EMPFT.RADS)                     *
C  *                  10 = STATS OUTPUT FILE (EMPFT.RES2)                     *
C  *                                                                          *
C  *    Protocol:                                                             *
C  *     1. Get circularly averaged reference average projection.             *
C  *     2. Cross-correlate raw images with projection average to get         *
C  *        estimate of particle center.                                      *
C  *     3. Compute polar map from raw image.                                 *
C  *     4. Compute polar FFT (PFT) of raw image.                             *
C  *     5. Cross-correlate raw PFT with data base of reference PFTs.         *
C  *     6. Cross-correlate raw image with "best" projection to resolve       *
C  *        PHI ambiquity and determine OMEGA.                                *
C  *                                                                          *
C  *    AUGUST 1993  PURDUE UNIVERSITY VAX 4000/90, T. BAKER.                 *
C  *    Modified May 1995 by D. Belnap & J. Conway, PFTs now R*4 in file      *
C  *    Modified 20 May 1995, D. Belnap, added GLOBAL search option           *
C  *    Modified July 1996, D. Belnap, added calc. and use of MAG & CC_CMP    *
C  *    Modified Oct 1996, D. Belnap, fixed bug in 2nd call to INTERP2D       *
C  *    Modified Dec 1996, D. Belnap, added J. Conway test for stray origins, *
C  *     moved PFTCC_FILL to MAIN, modified MAG search, minor fixes           *
C  *      Added 2-20 n-fold symmetries, 31 Jan 1997, D. Belnap                *
C  *      Changed format of output data file, 12 Feb 1997, D. Belnap          *
C  *    Added PIF format RW Ashmore & James Conway 5/1997                     *
C  *                                                                          *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    MODIFIED 24-DEC-1997 BY T. BAKER (Identify ARRAY pointers).           *
C  *    MODIFIED 19-JAN-1998 BY T. BAKER (Multiple parameter files).          *
C  *    MODIFIED  5-FEB-1998 BY T. BAKER (Use parameter arrays).              *
C  *    MODIFIED  6-MAR-1998 BY T. BAKER (Add UNIX compatibility).            *
C  *    MODIFIED 23-JUL-1999 BY TSB (Implement SBW's CTF code).               *
C  *    MODIFIED 25-AUG-1999 BY TSB (Implement TSB's CTF code).               *
C  *    MODIFIED 24-JUL-02 BY TSB/RWA (Add PRJS_GOLD array).                  *
C  *    LAST MODIFIED 25-OCT-2005 BY MBS, bug in XS(ID), YS(ID) calculations  *
C  ****************************************************************************

      SUBROUTINE GLOBAL_CC (MODE, BIN, NROT, NANNULI, NVIEW_MOD, 
     1     IDIM1, IDIM2, RES_MIN, RES_MAX, 
     2     PIXRES_MIN, PIXRES_MAX, ANN_LO, ANN_HI, JCUT, RADIUS,
     3     SYM, VERBOSE, MAG_CEN, MAG_STEP, MAG_NUM,
     4     MOD_TS, MOD_PS, IMG_BKG,
     5     PRJ_AVG, PRJS, PRJ, VAROPT, NCOL, NROW,
     6     NIDS_TOT, IMG_FNAMES, FIDS, IDS, TS, PS, OS, 
     7     XS, YS, MAGS, PFT_CCS, 
     8     PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC, SIGCUT, 
     9     XCEN_MODEL, YCEN_MODEL, PFTRAD_LO, PFTRAD_HI)

      IMPLICIT NONE

      include 'mpif.h'
      include '../include/allprogs.inc'

      INCLUDE '../include/pifhead.inc'
      TYPE (PIFHEAD_T) MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD

      COMMON/CTF_COM/PIXSIZ, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
     1 FOCUS_MIN, ANG_MAJ, CS_COEFF

      integer iii, mcw, idl, count
      integer worker, sendcount, basecount, remainder
      integer recvcounts(128), displs(128)
      double precision t2, t1, tg, ta, tb, t_ctf, t_tpo, t_xy
      double precision t_pfts, t_read, t_r4, t_ffta, t_fftb
      double precision t_chunk1, t_chunk2, t_chunk3, t_chunk4
      double precision t_chunk5, t_chunk6, t_bmag, t_list, t_asym

      INTEGER*4 MODE, BIN, I, J
      INTEGER*4 NCOL, NROW, NVIEW_MOD, ID
      INTEGER*4 NROT, NANNULI, IDIM1, IDIM2, JCUT
      INTEGER*4 NIMGS, NIDS_TOT, FILE_LAST, INDEX, UNITS(999)
      INTEGER*4 ANN_LO, ANN_HI, MAG_NUM, SYM, FILE, VERBOSE
      INTEGER*4 FIDS(NIDS_TOT), IDS(NIDS_TOT), COMPARE_MODE, CTFMODE
      INTEGER*4 FILTER
      real*4    IMG_BKG
      REAL*4    PIXSIZ(999), VOLTS(999), AMP_FAC(999), CS_COEFF(999)
      REAL*4    FOCUS_MAJ(999), FOCUS_MIN(999), ANG_MAJ(999)
      REAL*4    WAVELENGTH, WAVEL, TFAC, FILT_FAC(10), SIGCUT
      REAL*4    PFTRAD_LO, PFTRAD_HI

      REAL*4, allocatable :: R4_IMG(:,:), FFT_R4_IMG(:,:)
      REAL*4, allocatable :: PRJ_ROT(:,:), FFT_PRJ_ROT(:,:)
      REAL*4, allocatable :: FFT_PRJ_AVG(:,:)
      REAL*4, allocatable :: PFTS(:,:,:)
      REAL*4, allocatable :: PFFT(:,:),PMAP(:,:),FLTPMAP(:,:)
      REAL*4, allocatable :: PPRJ(:,:),PRJS_GOLD(:,:,:)
      REAL*4, allocatable :: TMP0(:,:),TMP1(:,:), TMP2(:,:)
      REAL*4, allocatable :: PFTIMG(:,:)
      REAL*4, allocatable :: PFT_AVGS(:), SQ2(:)
      REAL*4, allocatable :: RMS(:,:),AVER(:,:)
      real*4, allocatable :: IMG(:,:)
      real*4, allocatable :: GBUFFER(:)
      integer*4, allocatable :: IBUFFER(:)
      INTEGER*4, allocatable :: VAR(:,:)

      integer   flag, alloc_stat, status(mpi_status_size)
      REAL*4    TS(NIDS_TOT), PS(NIDS_TOT), OS(NIDS_TOT)
      REAL*4    XS(NIDS_TOT), YS(NIDS_TOT), MAGS(NIDS_TOT)
      REAL*4    PFT_CCS(NIDS_TOT), PRJ_CCS(NIDS_TOT), CMP_CCS(NIDS_TOT)
      real*4    peakx1(nids_tot), peaky1(nids_tot), step1(nids_tot)
      real*4    peakx2(nids_tot), peaky2(nids_tot), step2(nids_tot)
      integer *4 ipx(nids_tot), ipy(nids_tot)

      REAL*4    RADIAN, DEGREE
      REAL*4    DEL_ANG, DEL_PIXRES
      REAL*4    THE_BEST, PHI_BEST, OME_BEST, PIXRES_MIN, PIXRES_MAX
      REAL*4    AVG, SQ1, RADIUS, CC_PFT, CC_PRJ, CC_CMP
      REAL*4    XCEN, YCEN, XCEN_MODEL, YCEN_MODEL, RES_MIN, RES_MAX
      REAL*4    CC_RES_GLOBAL(2,20), CC_AVG, CC_RAD_GLOBAL(2,400)
      REAL*4    CC_RES_GLOBALL(2,20), CC_RAD_GLOBALL(2,400)
      REAL*4    BEST_MAG, MAG, MAG_STEP, MAG_CEN, MAG_LO
      REAL*4    MOD_TS(NVIEW_MOD), MOD_PS(NVIEW_MOD)
      REAL*4    PRJ_AVG(NCOL,NROW), PRJS(NCOL,NROW,NVIEW_MOD)
      REAL*4    FFT_ORIGX, FFT_ORIGY
      REAL*4    PRJ(NCOL,NROW)
      LOGICAL   FLIP, VAROPT, FIRST_TIME
      DATA      CC_RES_GLOBAL/40*0.0/, CC_RAD_GLOBAL/800*0.0/
      DATA      CC_RES_GLOBALL/40*0.0/, CC_RAD_GLOBALL/800*0.0/
      CHARACTER(len=256) IMG_FILENAME
      CHARACTER(len=256) IMG_FNAMES(999)
      CHARACTER READWRITE(4)
C     -------------------------------------------------------------------

C     ------------------------- EVERYBODY ---------------------------

      FFT_ORIGX = NCOL/2
      FFT_ORIGY = NROW/2

      t_ctf   = 0.0
      t_tpo   = 0.0
      t_xy    = 0.0
      t_pfts  = 0.0
      t_read  = 0.0
      t_r4    = 0.0
      t_ffta  = 0.0
      t_fftb = 0.0

      t_bmag = 0.0
      t_list = 0.0
      t_asym = 0.0

      t_chunk1 = 0.0
      t_chunk2 = 0.0
      t_chunk3 = 0.0
      t_chunk4 = 0.0
      t_chunk5 = 0.0
      t_chunk6 = 0.0

      t1 = mpi_wtime()
      ta = mpi_wtime()

      basecount = nids_tot/num_nodes
      remainder = nids_tot - basecount*num_nodes

      if(iam .ge. (num_nodes-remainder)) then
         sendcount = basecount + 1
      else
         sendcount = basecount
      endif

      if(iam .eq. 0) then
         recvcounts = 0
         displs = 0
         do iii=0,num_nodes-1
            if(iii .ge. (num_nodes-remainder)) then
               recvcounts(iii+1) = basecount + 1
            else
               recvcounts(iii+1) = basecount
            endif
         enddo
         displs(1) = 0
         do iii=1,num_nodes-1
            displs(iii+1) = displs(iii) + recvcounts(iii)
         enddo
      endif
      
      idl = 0 ! Local index

      allocate(prjs_gold(ncol,nrow,nview_mod),
     $     pprj(nrot,nannuli), 
     $     prj_rot(ncol,nrow),
     $     pft_avgs(nview_mod), 
     $     pfts(nrot/4,nannuli,nview_mod),
     $     pftimg(nrot/4,nannuli), 
     $     pfft(2*nrot,nannuli),
     $     pmap(nrot,nannuli), 
     $     fltpmap(nrot,nannuli), 
     $     sq2(nview_mod), 
     $     var(nrot/4,nannuli), 
     $     rms(nrot/4,nannuli), 
     $     aver(nrot/4,nannuli), 
     $     fft_r4_img(2*idim1,idim2),
     $     fft_prj_avg(2*idim1,idim2),
     $     fft_prj_rot(2*idim1,idim2),
     $     img(ncol,nrow), 
     $     r4_img(ncol,nrow), 
     $     tmp0(ncol,nrow), 
     $     tmp1(2*idim1,idim2),
     $     tmp2(2*idim1,idim2),
     $     gbuffer(nids_tot),
     $     ibuffer(nids_tot),
     $     stat = alloc_stat)

      if (alloc_stat .gt. 0) then
         write(*,*) 'could not allocate memory for prjs_gold etc.'
         call tsbend
      end if

      CALL PIRADDEG (PI, RADIAN, DEGREE)
      NIMGS      = 0
      DEL_PIXRES = 0.025
      DEL_ANG    = 360.0/FLOAT(NROT)
      PRJS_GOLD = PRJS ! Fortran 90 array syntax

      mcw = mpi_comm_world ! Abbreviation
      call mpi_bcast(PIXSIZ,    999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(VOLTS,     999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(AMP_FAC,   999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(CS_COEFF,  999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(FOCUS_MAJ, 999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(FOCUS_MIN, 999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(ANG_MAJ,   999, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(UNITS,     999, mpi_integer, 0, mcw, mpi_err)

      FILE_LAST  = FIDS(1)
      FIRST_TIME = .TRUE.

      tb = mpi_wtime()
      t_chunk1 = t_chunk1 + (tb - ta)

C     Loop through raw images as given in PARAMETER files 
      do i = 1, nids_tot, num_nodes
         do j = 0, min(num_nodes-1, nids_tot-i)
            
            id = i+j
            worker = num_nodes - 1 - j

            FILE = FIDS(ID)
C     If new micrograph then need to recalculate the PFTS database
            ta = mpi_wtime()
            IF(FIRST_TIME .OR. 
     $           (FILE.NE.FILE_LAST .AND. COMPARE_MODE == 1)) THEN
               FIRST_TIME = .FALSE.

               if(iam .eq. 0) then
                  WRITE(*,*) ' Computing PFT database ',
     +                 'for micrograph', IMG_FNAMES(FILE)
               endif

               CALL CALC_PFTS_G (FILE, NVIEW_MOD, NCOL, NROW, NROT,
     $              NANNULI, PRJ, PRJ_AVG, PMAP, XCEN_MODEL,
     $              YCEN_MODEL, PFTRAD_LO, PFTRAD_HI, DEL_ANG, PFFT,
     $              PFTS, PRJS, PRJS_GOLD, COMPARE_MODE, IDIM1,
     $              IDIM2, BIN, PI, RADIAN)
C     Compute FFT of circularly averaged reference
               IF(ABS(MODE).EQ.1) THEN
                  CALL PRJAVG_FFT (PRJ_AVG, NCOL, NROW, NVIEW_MOD,
     $                 FFT_PRJ_AVG,IDIM1, IDIM2, MODE, PRJHEAD)
               ENDIF
C     Load PFTs, theta and phi arrays
               CALL PFTCC_FILL_G (PFTS, PFT_AVGS, NROT, NANNULI,
     $              NVIEW_MOD,SQ2, JCUT, PIXRES_MIN, PIXRES_MAX,
     $              ANN_LO, ANN_HI, VAR, RMS,AVER, VAROPT, SIGCUT)
            ENDIF
            tb = mpi_wtime()
            t_pfts = t_pfts + (tb - ta)

C     --------------------------- MASTER ----------------------------
            ta = mpi_wtime()
            if(iam .eq. 0) then
                              
               IMG_FILENAME = IMG_FNAMES(FILE)
               
C     Open the correct image data file
               IMGHEAD%LUNUM = 7 ! UNIT FOR INPUT IMAGE FILE
               READWRITE(1)  = 'r'
               IF(ID.EQ.1) THEN
                  CALL PIF_OPEN (IMG_FILENAME, IMGHEAD%LUNUM, READWRITE)
                  CALL PIF_READ_GH (IMGHEAD)
               ENDIF
               IF(FILE_LAST.NE.FILE) THEN ! CLOSE CURRENT IMAGE FILE
                  CALL PIF_CLOSE (IMGHEAD%LUNUM)
                  CALL PIF_OPEN (IMG_FILENAME, IMGHEAD%LUNUM, READWRITE)
                  CALL PIF_READ_GH (IMGHEAD)
                  FILE_LAST = FILE
               ENDIF
               
               CC_AVG = 0.0
               NIMGS  = NIMGS + 1
               
C     Read in raw IMG and convert to floating point format
               IMGHEAD%IMGNUM = IDS(ID) ! SPECIFY WHICH "IMG" YOU WANT
               CALL PIF_READ_DH (IMGHEAD)
               IMG_BKG = IMGHEAD%IMG_BKG
               if(imgHead%mode .eq. 46) then
                  CALL PIF_READ_IMGI4(IMGHEAD, IMG, NCOL, NROW, IDS(ID))
               else
                  print *, 'Invalid PIF mode - ', imgHead%mode
                  call tsbend
               endif

C     Reads for now invalid modes
C     CALL PIF_READ_IMGI1 (IMGHEAD, IMG, NCOL, NROW, IDS(ID)) !mode==6
C     CALL PIF_READ_IMGI2 (IMGHEAD, IMG, NCOL, NROW, IDS(ID)) !mode==7
               
               if(worker .ne. 0) then
                  call mpi_send(img, ncol*nrow,
     $                 mpi_real,    worker, 104, mcw, mpi_err)
                  call mpi_send(img_bkg, 1,
     $                 mpi_real,    worker, 105, mcw, mpi_err)
               endif

            endif
            tb = mpi_wtime()
            t_read = t_read + (tb - ta)
            
C     --------------------------- WORKER ----------------------------
            if(iam .eq. worker) then

               IF(FILE_LAST.NE.FILE) THEN
                  FILE_LAST = FILE
               ENDIF
               
               idl = idl + 1

               if(worker .ne. 0) then
                  call mpi_recv(img, ncol*nrow,
     $                 mpi_real,    0, 104, mcw, status, mpi_err)
                  call mpi_recv(img_bkg, 1,
     $                 mpi_real,    0, 105, mcw, status, mpi_err)
               endif

C     Low-pass Fourier filter the image to help TPO calculation
C     This replace the IMG_MAP call.
               ta = mpi_wtime()
               CALL MAP_FFT_FILL (IMG, NCOL, NROW, FFT_R4_IMG,
     $              IDIM1, IDIM2, IMG_BKG)
               CALL FFT_2D (FFT_R4_IMG, IDIM1, IDIM2)
               tb = mpi_wtime()
               t_r4 = t_r4 + (tb - ta)

C     For compare mode == 3, divide raw imges by CTF
               IF(COMPARE_MODE.EQ.3) THEN
                  CTFMODE = 1
                  FILTER  = 2
                  TFAC    = 0.0 ! Hardwired for now 20-Jun-02
C     Formerly call to CTF_DIVIDE; CC,BETA,De set TO 0.0
                  WAVELENGTH = WAVEL(VOLTS(FILE))
                  ta = mpi_wtime()
                  CALL CTF_SCALE (CTFMODE, FILTER, FFT_R4_IMG, 
     $                 IDIM1, IDIM2,
     $                 UNITS(FILE), BIN*PIXSIZ(FILE), PI, RADIAN,
     $                 WAVELENGTH, FOCUS_MAJ(FILE), FOCUS_MIN(FILE),
     $                 ANG_MAJ(FILE), AMP_FAC(FILE), CS_COEFF(FILE),
     $                 FILT_FAC, TFAC, .TRUE., 0.0, 0.0, 0.0,
     $                 VOLTS(FILE)) 
                  tb = mpi_wtime()
                  t_ctf = t_ctf + (tb - ta)
               ENDIF
               
               ta = mpi_wtime()
               CALL FFT_LOP (FFT_R4_IMG, IDIM1, IDIM2, RES_MAX, 
     $              2.0, .TRUE.)
               CALL FFT_2D_BACK (FFT_R4_IMG, IDIM1, IDIM2)
               
C     Store modified version of image into R4_IMG and IMG
               CALL FFT_MAP_FILL (FFT_R4_IMG, IDIM1, IDIM2, R4_IMG, 
     $              NCOL, NROW, 0.0)
               CALL FFT_MAP_FILL (FFT_R4_IMG, IDIM1, IDIM2, IMG,
     $              NCOL, NROW, IMG_BKG)
               tb = mpi_wtime()
               t_ffta = t_ffta + (tb - ta)
               
               
C     For compare_mode == 2, apply CTF to FFT to downweight data
C     according to Steve Fuller scheme
               ta = mpi_wtime()
               IF(COMPARE_MODE.EQ.2) THEN
                  CALL MAP_FFT_FILL (R4_IMG, NCOL, NROW, FFT_R4_IMG,
     $                 IDIM1,IDIM2, 0.0)
                  CALL FFT_2D (FFT_R4_IMG, IDIM1, IDIM2)
                  WAVELENGTH = WAVEL(VOLTS(FILE))
                  TFAC    = 0.0 ! HARDWIRE THIS FOR NOW
                  
C     2-May-00 Hardwire input parameters to CTF_MULTIPLY such as
C     mode=1; Cc=0.0, BETA=0.0, dE=0.0; These will *not* impose
C     a decay factor on the data.
                  
C     Fomerly call to CTF_MULTIPLY !RSS
                  CALL CTF_SCALE (1, 0, FFT_R4_IMG, IDIM1, IDIM2, 
     $                 UNITS(FILE),
     $                 BIN*PIXSIZ(FILE), PI, RADIAN, WAVELENGTH,
     $                 FOCUS_MAJ(FILE), FOCUS_MIN(FILE), ANG_MAJ(FILE),
     $                 AMP_FAC(FILE), CS_COEFF(FILE), FILT_FAC, TFAC,
     $                 .TRUE., 0.0, 0.0, 0.0, VOLTS(FILE))
                  
                  CALL FFT_2D_BACK (FFT_R4_IMG, IDIM1, IDIM2)
                  CALL FFT_MAP_FILL (FFT_R4_IMG, IDIM1, IDIM2, R4_IMG,
     $                 NCOL, NROW, 0.0)
                  WRITE(*,'(A,I4,A,2F8.4,A,F8.4)')
     $                 'Applied CTF to image ',ID,
     $                 ' with defocus values', FOCUS_MAJ(FILE)
     $                 ,FOCUS_MIN(FILE), ' and ANG_MAJ =', ANG_MAJ(FILE)
               ENDIF
               tb = mpi_wtime()
               t_chunk2 = t_chunk2 + (tb - ta)

C     ********************************************************************
C     *   I. Get initial MAG scale factor (1.0 is default)               *
C     *      A) GLOBAL search mode, initial sc. factor = MAG_CEN         *
C     *      B) GLOBAL mode, read from input parameter file              *
C     *  II. Get initial X,Y origins                                     *
C     *      A) ABS(MODE).EQ.1, estimate by cross-correlation with cir-  *
C     *         cularly averaged reference, RES_MIN=1.0 (no choice)      *
C     *      B) ABS(MODE).EQ.2, read from input parameter file.          *
C     * III. If MAG does not equal 1.0, scale image by MAG               *
C     *  IV. Compute FFT of image                                        *
C     ********************************************************************
               
               ta = mpi_wtime()
               CALL MAP_FFT_FILL (R4_IMG, NCOL, NROW, FFT_R4_IMG,
     $              IDIM1, IDIM2, 0.0)
               CALL FFT_2D (FFT_R4_IMG, IDIM1, IDIM2)
               tb = mpi_wtime()
               tb = t_fftb + (tb - ta)
               
               ta = mpi_wtime()
               IF(ABS(MODE).EQ.1) THEN ! Global mode #1 (no origins)
                  CALL GET_XY (FFT_R4_IMG, FFT_PRJ_AVG, XCEN, YCEN,
     1                 IDIM1, IDIM2, RES_MAX, 
     1                 FFT_ORIGX, FFT_ORIGY, flag,
     2                 ipx(idl), ipy(idl), peakx1(idl), peakx2(idl),
     3                 peaky1(idl), peaky2(idl), step1(idl), step2(idl))

                  if(flag .lt. 0) then
                     print *, 'WARNING:  Center is more than 30% off'
                     print *, '          Particle #', id
                  endif
                  MAG = MAG_CEN
               ELSE             ! Global mode #2 (with origins)
                  XCEN = ((XS(ID) + 1.0)/BIN) - 1.0 ! NEED IN BINNED FFT SPACE
                  YCEN = ((YS(ID) + 1.0)/BIN) - 1.0 ! NEED IN BINNED FFT SPACE
                  MAG  = MAGS(ID)
               ENDIF
               tb = mpi_wtime()
               t_chunk3 = t_chunk3 + (tb - ta)
               
c     Mag .ne. 0 code can be found in versions 1.16 and earlier of this file
               if(mag .ne. 1.0) then
                  print *, 'MAG not equal to 1.0 ignored'
               endif
               
               XCEN = XCEN + 1.0 ! Get back into real space
               YCEN = YCEN + 1.0
               
C     *******************************************************************
C     *  Sometimes EMPFT crashes with a division-by-0 error in          *
C     *  GET_THEPHI (called below). It turns out that GET_XY            *
C     *  (above) returns a bad origin due to poor matching between an   *
C     *  image and its reference, and then MAP_POLAR (below) fills the  *
C     *  polar array with zero since the origin can be too far off the  *
C     *  edge of the cartesian array. The div-by-0 occurs because the   *
C     *  sum-of-sqaures (SQ1) for the polar array is zero.  Check here  * 
C     *  for (XCEN,YCEN) being away from the center of the image array  *
C     *  (NCOL/2,NROW/2) and post warnings. If it is way off, the user  *
C     *  can try to skip this particle and go on to the next, rather    *
C     *  than crash in GET_THEPHI.                                      *
C     *******************************************************************
               
               IF (ABS(XCEN-FLOAT(NCOL/2)).GT.FLOAT(NCOL/2) .OR. 
     1              ABS(YCEN-FLOAT(NROW/2)).GT.FLOAT(NROW/2)) THEN
C     WRITE (*,410) IDS(ID), XCEN*BIN, YCEN*BIN
               ELSE IF (ABS(XCEN-FLOAT(NCOL/2)).GT.FLOAT(NCOL/10) .OR.
     1                 ABS(YCEN-FLOAT(NROW/2)).GT.FLOAT(NROW/10)) THEN
                  WRITE (*,415) IDS(ID), XCEN*BIN, YCEN*BIN
               ENDIF
               
 410           format (' WARNING - estimated center is off ',
     $              'edge of array:',/,
     $              ' img_num, xcen, ycen = ', i6, 2f10.3)
 415           format (' WARNING - estimated center is more ',
     $              'than 10% from array center:',/,
     $              ' img_num, xcen, ycen = ', i6, 2f10.3)

               ta = mpi_wtime()
               CALL GET_TPO_G (MODE, NCOL, NROW, NROT, NANNULI,
     $              NVIEW_MOD,JCUT, XCEN, YCEN, XCEN_MODEL,
     $              YCEN_MODEL, RADIUS,DEL_ANG, PIXRES_MIN, PIXRES_MAX,
     $              ANN_LO, ANN_HI, VAR,VAROPT, PI, AVG, SQ1, SQ2,
     $              MOD_TS, MOD_PS, THE_BEST,PHI_BEST, OME_BEST, PFTS,
     $              PFT_AVGS, PRJS,CC_PFT, CC_PRJ, R4_IMG, PMAP, PFFT,
     $              PRJ, PPRJ, PFTIMG, FLIP, INDEX)
               tb = mpi_wtime()
               t_tpo = t_tpo + (tb - ta)

               IF(FLIP) THEN
                  THE_BEST = 180.0 - THE_BEST
                  IF(THE_BEST.EQ.0.0 .OR. THE_BEST.EQ.180.0) THEN
                     PHI_BEST = 0.0
                  ELSE
                     PHI_BEST = PHI_BEST + 180.0
                  ENDIF
                  IF(THE_BEST.NE.180.0)  OME_BEST = OME_BEST + 180.0
                  CALL MAP_XFLIP (PRJ, NCOL, NROW)
               ENDIF
               
C     Refine X,Y origin
               ta = mpi_wtime()
               PRJ_ROT = PRJ    !Fortran 90 array syntax
               IF (OME_BEST.NE.0.0) THEN
                  CALL INTERP2D (PRJ_ROT, NCOL, NROW, XCEN_MODEL-1.0,
     $                 YCEN_MODEL-1.0, XCEN_MODEL-1.0, YCEN_MODEL-1.0,
     $                 OME_BEST, 1.0, 0.0)
               ENDIF
               CALL MAP_FFT_FILL(PRJ_ROT, NCOL, NROW, FFT_PRJ_ROT,
     $              IDIM1, IDIM2, 0.0)
               CALL FFT_2D (FFT_PRJ_ROT, IDIM1, IDIM2)
               tb = mpi_wtime()
               t_chunk4 = t_chunk4 + (tb - ta)
               
               ta = mpi_wtime()
               CALL GET_XY (FFT_R4_IMG, FFT_PRJ_ROT, XCEN, YCEN, 
     $              IDIM1, IDIM2,
     $              RES_MAX,FFT_ORIGX, FFT_ORIGY, flag,
     2              ipx(idl), ipy(idl), peakx1(idl), peakx2(idl),
     3              peaky1(idl), peaky2(idl), step1(idl), step2(idl))
               tb = mpi_wtime()
               t_xy = t_xy + (tb - ta)

               if(flag .lt. 0) then
                  print *, ' WARNING:  Center is more than 30% off'
                  print *, '          Particle #', id
                  print *, '          flag', flag
               endif
               XCEN = XCEN + 1.0 ! Get back into real space
               YCEN = YCEN + 1.0
               
C     Test centers to determine whether they are off the Cartesian grid
C     or more than 10% from the array center. These may be indicative of
C     poor data.
               
               IF (ABS(XCEN-FLOAT(NCOL/2)).GT.FLOAT(NCOL/2) .OR. 
     1              ABS(YCEN-FLOAT(NROW/2)).GT.FLOAT(NROW/2))  THEN
                  WRITE (*,410) IDS(ID), XCEN*BIN, YCEN*BIN
               ELSE IF (ABS(XCEN-FLOAT(NCOL/2)).GT.FLOAT(NCOL/10) .OR.
     1                 ABS(YCEN-FLOAT(NROW/2)).GT.FLOAT(NROW/10))  THEN
                  WRITE (*,415) IDS(ID), XCEN*BIN, YCEN*BIN
               ENDIF
               
C     Find/refine MAG scale factor and CMP coefficient, use a routine
C     adapted from CMP program, IMG and PRJ are compared in real space
               
               IF (MAG_CEN.LE.0.0) THEN
                  MAG_LO = MAG     - (((MAG_NUM-1)/2)*MAG_STEP)
               ELSE
                  MAG_LO = MAG_CEN - (((MAG_NUM-1)/2)*MAG_STEP)
               ENDIF
               
C     RWA July 2004 - This is different???  e.g. We're just doing statistics
C     on MAG here???
               
               ta = mpi_wtime()
               CALL GET_BESTMAG(IMG, PRJ, TMP0, TMP1, NCOL, NROW,
     $              IDIM1,IDIM2, REAL(ANN_LO), REAL(ANN_HI), RES_MIN,
     $              RES_MAX,XCEN-1.0,YCEN-1.0, OME_BEST, MAG_LO,
     $              MAG_NUM, MAG_STEP,BEST_MAG, CC_CMP,IMG_BKG)
               
               MAG    = BEST_MAG
               tb = mpi_wtime()
               t_bmag = t_bmag + (tb-ta)
               
C     IF VERBOSE > 0                                              
C     RECOMPUTE POLAR MAP OF RAW IMAGE WITH NEWEST X,Y ORIGIN
C     RECOMPUTE POLAR MAP OF ROTATED PROJECTION
C     COMPUTE COEFFICIENTS AS A FUNCTION OF RESOLUTION AND RADIUS
C     WRITE TO EXTRA OUTPUT FILES (UNITS 8,9,10)
               
               ta = mpi_wtime()
               IF(VERBOSE.GT.0) then
                  CALL LIST_CCS(BIN, NCOL, NROW, IDIM1, IDIM2, NROT,
     $                 NANNULI, IMG, TMP0, R4_IMG, 
     $                 FFT_R4_IMG, FFT_PRJ_ROT, TMP1, TMP2, 
     $                 PRJ_ROT, PPRJ, PMAP, FLTPMAP, IDS(ID),
     $                 IMG_BKG, XCEN,YCEN, XCEN_MODEL, YCEN_MODEL,
     $                 RES_MIN, RES_MAX, ANN_LO,
     $                 ANN_HI, RADIUS, PIXRES_MAX,DEL_PIXRES, DEL_ANG,
     $                 CC_AVG, CC_RES_GLOBALL,CC_RAD_GLOBALL, verbose)
               ENDIF
               tb = mpi_wtime()
               t_list = t_list + (tb-ta)
               
C     Return orientation to asymmetric unit
               ta = mpi_wtime()
               IF (SYM.NE.1) THEN
                  CALL TO_ASYM_UNIT (SYM, THE_BEST, PHI_BEST, OME_BEST)
               ENDIF
               tb = mpi_wtime()
               t_asym = t_asym + (tb-ta)

C     Make sure that omega stays in the range 0 to 360
               ta = mpi_wtime()
               IF (OME_BEST.LT.0.0) OME_BEST = OME_BEST + 360.0
               IF (OME_BEST.ge.360.0) OME_BEST = OME_BEST - 360.0
               
               XS(IDL)      = (XCEN - 1.0)*BIN !MBS 10/25/05
               YS(IDL)      = (YCEN - 1.0)*BIN !MBS 10/25/05
               TS(IDL)      = THE_BEST
               PS(IDL)      = PHI_BEST
               OS(IDL)      = OME_BEST
               MAGS(IDL)    = MAG
               PFT_CCS(IDL) = CC_PFT
               PRJ_CCS(IDL) = CC_PRJ
               CMP_CCS(IDL) = CC_CMP
               tb = mpi_wtime()
               t_chunk5 = t_chunk5 + (tb - ta)

            endif
            
         enddo
      enddo
      
      ta = mpi_wtime()
      call mpi_barrier(mcw, mpi_err)

      call mpi_reduce(cc_res_globall, cc_res_global, 40,  mpi_real,
     +     mpi_sum, 0, mpi_comm_world, mpi_err)
      call mpi_reduce(cc_rad_globall, cc_rad_global, 800, mpi_real,
     +     mpi_sum, 0, mpi_comm_world, mpi_err)

      call mpi_gatherv(peakx1, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               peakx1(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(peakx2, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               peakx2(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(peaky1, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               peaky1(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(peaky2, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               peaky2(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(step1, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               step1(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(step2, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               step2(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(ipx, sendcount, mpi_integer, 
     +     ibuffer, recvcounts, displs, mpi_integer,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               ipx(iii) = ibuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(ipy, sendcount, mpi_integer, 
     +     ibuffer, recvcounts, displs, mpi_integer,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               ipy(iii) = ibuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(xs, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               xs(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(ys, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               ys(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(ts, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               ts(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(ps, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               ps(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(os, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               os(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(mags, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               mags(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(pft_ccs, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               pft_ccs(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(prj_ccs, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               prj_ccs(iii) = gbuffer(count)
            enddo
         enddo
      endif

      call mpi_gatherv(cmp_ccs, sendcount, mpi_real, 
     +     gbuffer, recvcounts, displs, mpi_real,
     +     0, mcw, mpi_err)
      if(iam .eq. 0) then
         count = 0
         do i=1, num_nodes
            do j=1,recvcounts(i)
               count = count + 1
               iii = (num_nodes - i + 1) + (j-1)*num_nodes
               cmp_ccs(iii) = gbuffer(count)
            enddo
         enddo
      endif
      tb = mpi_wtime()
      t_chunk6 = t_chunk6 + (tb - ta)

C     --------------------------- MASTER ----------------------------
C     **************************************************************
C     *  Write extra output to files if user wishes (VERBOSE > 0)  *
C     *   UNIT 8 = STATS OUTPUT FILE (EMPFT.RES1)                  *
C     *        9 = STATS OUTPUT FILE (EMPFT.RADS)                  *
C     *       10 = STATS OUTPUT FILE (EMPFT.RES2)                  *
C     **************************************************************
      
      if(iam .eq. 0) then

         do id = 1, nids_tot
            write(*,'(a,2i6)')"Best origin by simple search    :",
     $           IPX(id),IPY(id)
            write(*,'(a,3f12.4)')"Best origin by surface fit, step:",
     $           PEAKX1(id),PEAKY1(id),step1(id)
            write(*,'(a,3f12.4)')"Best origin by surface fit, step:",
     $           PEAKX2(id),PEAKY2(id),step2(id)
            WRITE(*,1110) IDS(ID), TS(ID), PS(ID), OS(ID), XS(ID),
     $           YS(ID),MAGS(ID), PFT_CCS(ID), PRJ_CCS(ID),
     $           CMP_CCS(ID)
 1110       FORMAT (1X,I6,',',4(F8.3,','),F7.3,',',3(F6.3,','),F6.3)
         enddo

         IF (VERBOSE.GE.1) THEN
            J = NINT(1/(PIXRES_MAX*DEL_PIXRES))
            WRITE(8,1209)
            WRITE(8,'(A,10F7.3)') ' AVG', 
     1           (CC_RES_GLOBAL(1,I)/FLOAT(NIMGS),I=1,J)
            WRITE(10,1209)
            WRITE(10,'(A,10F7.3)') ' AVG', 
     1           (CC_RES_GLOBAL(2,I)/FLOAT(NIMGS), I=1,J)
            WRITE(9,1210) NIMGS
C     a Blow out all data even past ann_hi
C     a ORIG way just to ann_hi         DO I = 1,ANN_HI+1
            DO I = 1,nannuli
               J = (I-1)*BIN
               WRITE(9,1211) J, CC_RAD_GLOBAL(1,I)/FLOAT(NIMGS), 
     1              CC_RAD_GLOBAL(2,I)/FLOAT(NIMGS)
            ENDDO
         ENDIF
 1209    FORMAT(74('-')) 
 1210    FORMAT (' AVERAGE correlation coefficients as function ',
     $        'of radius for', I6,' images'/,' CCs are for unfiltered/',
     $        'filtered IMG vs. POLAR_PRJ',/
     $        ,' Radius Unfiltered Filtered', /
     $        ,'---------------------------')
 1211    FORMAT(I5,2F10.3)
      endif
      
C     ------------------------- EVERYBODY ---------------------------
      deallocate(prjs_gold, pprj, prj_rot, pft_avgs, pfts,
     $     pftimg, pfft, pmap, fltpmap, sq2, var, rms,
     $     aver, fft_r4_img, fft_prj_avg, fft_prj_rot, img, r4_img,
     $     tmp0, tmp1, tmp2, gbuffer, ibuffer)
      
      t2 = mpi_wtime()
      tg = t2-t1
      if(iam.eq.0) then
         write(*,*) 'Global_cc time = ',tg, ' NCPUs = ', num_nodes
         write(*,*) '  CTF time     = ',t_ctf
         write(*,*) '  TPO time     = ',t_tpo
         write(*,*) '   XY time     = ',t_xy
         write(*,*) ' PFTS time     = ',t_pfts
         write(*,*) ' READ time     = ',t_read
         write(*,*) '   R4 time     = ',t_r4
         write(*,*) ' FFTA time     = ',t_ffta
         write(*,*) ' FFTB time     = ',t_fftb
         write(*,*) '   C1 time     = ',t_chunk1
         write(*,*) '   C2 time     = ',t_chunk2
         write(*,*) '   C3 time     = ',t_chunk3
         write(*,*) '   C4 time     = ',t_chunk4
         write(*,*) '   C5 time     = ',t_chunk5
         write(*,*) ' BMAG time     = ',t_bmag
         write(*,*) ' LIST time     = ',t_list
         write(*,*) ' ASYM time     = ',t_asym
         write(*,*) '   C6 time     = ',t_chunk6
      endif

      RETURN
      END subroutine global_cc
