      subroutine comp_sfactor(iam, f_1, f_2, hkl_max, 
     +     first_iq, last_iq, res_max, res_min,
     +     nrffta_row, pixelsize, res_step, mem_cmpt)

C     Calculates the structure factors from the 3D FTs (f_1 and f_2) of
C     two maps. Typically these correspond to the even and odd maps
C     generated by the program P3DR. Since this routine writes out the
C     results, no values are returned to the calling program except for
C     the memory usage (mem_cmpt).
      
      implicit none
      
      include 'mpif.h'

      integer, intent(in) :: nrffta_row, res_step, hkl_max, iam
      integer, intent(in) :: first_iq, last_iq
      real,    intent(in) :: res_max, res_min, pixelsize
      complex, intent(in) :: f_1(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq)
      complex, intent(in) :: f_2(-hkl_max:hkl_max, -hkl_max:hkl_max,
     +     first_iq:last_iq)

      double precision, intent(out) :: mem_cmpt
      
C     Local variables
      REAL      res(res_step) 
      
      INTEGER   NTOT(res_step), sNTOT(res_step)
      DOUBLE PRECISION    F1SUM(res_step), sF1SUM(res_step),
     +     F2SUM(res_step), sF2SUM(res_step),
     +     SUMDAB(res_step), sSUMDAB(res_step),
     +     AVG_PDIF(res_step), sAVG_PDIF(res_step)
      DOUBLE PRECISION    F1SQSUM(res_step), sF1SQSUM(res_step),
     +     F2SQSUM(res_step), sF2SQSUM(res_step),
     +     F1_F2SUM(res_step), sF1_F2SUM(res_step),
     +     F1F2SUM(res_step), sF1F2SUM(res_step)
      
      REAL    CCOEF(res_step) , RFAC, Rphase, rad_to_deg
      
      integer h,k,l,i,bin, mpi_err
      real max_annulsq, min_annulsq, dstarsq, res_inc
      real PDIF, DENOM1, DENOM2, PHASE1, PHASE2
      real APART1, APART2, BPART1, BPART2, AMP1, AMP2

      rad_to_deg = 180./ (4.0 * atan(1.0))
      
      mem_cmpt =  res_step * ( 12*4 + 8*8)
      max_annulsq = (real(nrffta_row)*pixelsize/res_max)**3
      min_annulsq = (real(nrffta_row)*pixelsize/res_min)**3
      res_inc   = (max_annulsq - min_annulsq)/res_step
      
      DO I = 1,res_step         ! INITIALIZE ARRAYS
         res(I)=nrffta_row*pixelsize/((min_annulsq+i*res_inc)**(1./3))
         NTOT(I)     =   0
         AVG_PDIF(I) = 0.0
         F1_F2SUM(I) = 0.0
         F1SUM(I)    = 0.0
         F2SUM(I)    = 0.0
         F1F2SUM(I)  = 0.0
         F1SQSUM(I)  = 0.0
         F2SQSUM(I)  = 0.0
         SUMDAB(I)    = 0.0
      ENDDO
      
      do l = first_iq, last_iq
         do k = -hkl_max, hkl_max
            do h = -hkl_max, hkl_max
               dstarsq = (sqrt(real(h*h + k*k + l*l)))**3
               if(dstarsq .GE. min_annulsq .and. 
     +              dstarsq .LE. max_annulsq)then
                  bin = (dstarsq - min_annulsq)/res_inc + 1  
                  if (bin .gt. res_step) bin = res_step
                  
                  CALL cplex2AP(f_1(h,k,l), APART1, BPART1,
     +                 AMP1, PHASE1, rad_to_deg)
                  CALL cplex2AP(f_2(h,k,l), APART2, BPART2,
     +                 AMP2, PHASE2, rad_to_deg)
                  
                  PDIF   = ABS(PHASE1 - PHASE2)
                  IF(PDIF .GT. 180.0) PDIF = 360.0 - PDIF
                  AVG_PDIF(BIN) = AVG_PDIF(BIN) + PDIF
                  F1_F2SUM(BIN) = F1_F2SUM(BIN) + ABS(AMP1 - AMP2)
                  F1SUM(BIN)    = F1SUM(BIN) + AMP1
                  F2SUM(BIN)    = F2SUM(BIN) + AMP2
                  F1SQSUM(BIN)  = F1SQSUM(BIN) + AMP1*AMP1
                  F2SQSUM(BIN)  = F2SQSUM(BIN) + AMP2*AMP2
                  F1F2SUM(BIN)  = F1F2SUM(BIN) + AMP1*AMP2
                  SUMDAB(BIN)   = SUMDAB(BIN) + 
     +                 sqrt((APART1 - APART2)**2 + (BPART1 - BPART2)**2)
                  NTOT(BIN)     = NTOT(BIN) + 1
               endif
            enddo
         enddo
      enddo
      
      call mpi_reduce(NTOT, sNTOT, res_step, MPI_INTEGER, 
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(AVG_PDIF, sAVG_PDIF, res_step, 
     +     MPI_DOUBLE_PRECISION,MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1_F2SUM, sF1_F2SUM, res_step, 
     +     MPI_DOUBLE_PRECISION,MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1SUM, sF1SUM, res_step, MPI_DOUBLE_PRECISION,
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F2SUM, sF2SUM, res_step,MPI_DOUBLE_PRECISION, 
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1SQSUM, sF1SQSUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F2SQSUM, sF2SQSUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(F1F2SUM, sF1F2SUM, res_step, 
     +     MPI_DOUBLE_PRECISION, MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      call mpi_reduce(SUMDAB,sSUMDAB,res_step,MPI_DOUBLE_PRECISION,
     +     MPI_SUM, 0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop('PSF','comp_sfactor',
     +     'Error in calling MPI_reduce')
      
      if (iam.eq.0) then
         Write(*,'(/,2A)') '   Bin#  Resolution  observ  ',
     +        '   R-SF       CC     R-phase  phase-diff',
     +        '  ------  --------  -------- ', 
     +        ' --------  --------  --------  -------- '
         WRITE(*,'(I8, F10.2, I10, 4F10.3)') 0, res_min, 0, 0.0, 
     +        1.0, 0.0, 0.0
         
         DO I = 1, res_step
            CCOEF(I) = 1.0
            IF(sNTOT(I).GT.0) THEN
               DENOM1    = sF1SQSUM(I) - (sF1SUM(I)**2)/sNTOT(I)
               DENOM2   = sF2SQSUM(I) - (sF2SUM(I)**2)/sNTOT(I)
               IF(DENOM1.GT.0.0 .AND. DENOM2.GT.0.0) THEN
                  CCOEF(I) = (sF1F2SUM(I) - 
     +                 (sF1SUM(I)*sF2SUM(I)/sNTOT(I)))/
     +                 ( SQRT(DENOM1) * SQRT(DENOM2) )
               ENDIF
               RFAC     = sF1_F2SUM(I)/sF1SUM(I)
               Rphase   = sSUMDAB(I)/ sF1SUM(I)
               AVG_PDIF(I) = sAVG_PDIF(I)/REAL(sNTOT(I))
            ENDIF
            
            WRITE(*,'(I8, F10.2, I10, 4F10.3)') I, res(I), sNTOT(I), 
     +           RFAC, CCOEF(I), Rphase, AVG_PDIF(I)
         ENDDO
      endif
      
C     R-SF = sum(F1-kF2) / sum F1
C     S1 = sum(<F1>-F1)(<F2>-F2)
C     S2 = sum(<F1>-F1)^2
C     S3 = sum(<F2>-F2)^2
C     CC  = S1 / sqrt(S2*S3)
C     A = F1 cos(phi1*deg_to_rad) - F2 cos(phi2*deg_to_rad)
C     B = F2 sin(phi2*deg_to_rad) - F1 sin(phi1*deg_to_rad)
C     R-phase =  sum(sqrt(A^2+B^2)) / sum(f1)
C     phase-diff = sum |phi1-phi2| / # observations

      end subroutine comp_sfactor


      subroutine cplex2AP(c1, APART, BPART, AMP, PHI, rad_to_deg)

C     Given a complex variable, returns the real component, imaginary
C     component, amplitude, and phase.
C
C     Input:
C     c1 - complex variable
C     rad_to_deg - factor for converting radians to degrees
C
C     Output:
C     apart - real component of c1
C     bpart - imaginary component of c1
C     amp - amplitude of c1
C     phi - phase of c1 (in degrees)

      implicit none
      Complex, intent(in) :: c1
      real, intent(in)    :: rad_to_deg
      REAL, intent(out)   ::  AMP, PHI, APART, BPART
      
      APART = real(c1)
      BPART = aimag(c1)
      AMP   = SQRT(APART**2 + BPART**2)
      PHI   = 0.0
      IF(APART.EQ.0.0) THEN
         IF(BPART.GT.0.0) PHI =  90.0
         IF(BPART.LT.0.0) PHI = 270.0
      ELSE
         PHI = ATAN2(BPART,APART)*RAD_TO_DEG
         PHI = AMOD(PHI,360.0)
         IF(PHI.LT.0.0) PHI = 360.0 + PHI
      ENDIF

      return
      end subroutine cplex2AP
